from django.http import HttpResponse
from django.test import RequestFactory

from csp.middleware import CSPMiddleware
from csp.context_processors import nonce

rf = RequestFactory()
mw = CSPMiddleware()


def test_nonce_context_processor():
    request = rf.get('/')
    mw.process_request(request)
    context = nonce(request)

    response = HttpResponse()
    mw.process_response(request, response)

    assert context['CSP_NONCE'] == request.csp_nonce


def test_nonce_context_processor_with_middleware_disabled():
    request = rf.get('/')
    context = nonce(request)

    assert context['CSP_NONCE'] == ''



from django.template import engines, Template, Context
from django.test import RequestFactory

from csp.middleware import CSPMiddleware


JINJA_ENV = engines['jinja2']
mw = CSPMiddleware()
rf = RequestFactory()


class ScriptTestBase(object):
    def assert_template_eq(self, tpl1, tpl2):
        aaa = tpl1.replace('\n', '').replace('  ', '')
        bbb = tpl2.replace('\n', '').replace('  ', '')
        assert aaa == bbb

    def process_templates(self, tpl, expected):
        request = rf.get('/')
        mw.process_request(request)
        ctx = self.make_context(request)
        return (self.make_template(tpl).render(ctx).strip(),
                expected.format(request.csp_nonce))


class ScriptTagTestBase(ScriptTestBase):
    def make_context(self, request):
        return Context({'request': request})

    def make_template(self, tpl):
        return Template(tpl)


class ScriptExtensionTestBase(ScriptTestBase):
    def make_context(self, request):
        return {'request': request}

    def make_template(self, tpl):
        return JINJA_ENV.from_string(tpl)



import django


CSP_REPORT_ONLY = False

CSP_INCLUDE_NONCE_IN = ['default-src']

DATABASES = {
    'default': {
        'NAME': 'test.db',
        'ENGINE': 'django.db.backends.sqlite3',
    }
}

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'csp',
)

SECRET_KEY = 'csp-test-key'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.jinja2.Jinja2',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'environment': 'csp.tests.environment.environment',
            'extensions': ['csp.extensions.NoncedScript']
            },
    },
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {},
    },
]


# Django >1.6 requires `setup` call to initialise apps framework
if hasattr(django, 'setup'):
    django.setup()



import copy
import re
import warnings

from collections import OrderedDict
from itertools import chain

from django.conf import settings
from django.utils.encoding import force_str


CHILD_SRC_DEPRECATION_WARNING = \
    'child-src is deprecated in CSP v3. Use frame-src and worker-src.'


def from_settings():
    return {
        # Fetch Directives
        'child-src': getattr(settings, 'CSP_CHILD_SRC', None),
        'connect-src': getattr(settings, 'CSP_CONNECT_SRC', None),
        'default-src': getattr(settings, 'CSP_DEFAULT_SRC', ["'self'"]),
        'script-src': getattr(settings, 'CSP_SCRIPT_SRC', None),
        'script-src-attr': getattr(settings, 'CSP_SCRIPT_SRC_ATTR', None),
        'script-src-elem': getattr(settings, 'CSP_SCRIPT_SRC_ELEM', None),
        'object-src': getattr(settings, 'CSP_OBJECT_SRC', None),
        'style-src': getattr(settings, 'CSP_STYLE_SRC', None),
        'style-src-attr': getattr(settings, 'CSP_STYLE_SRC_ATTR', None),
        'style-src-elem': getattr(settings, 'CSP_STYLE_SRC_ELEM', None),
        'font-src': getattr(settings, 'CSP_FONT_SRC', None),
        'frame-src': getattr(settings, 'CSP_FRAME_SRC', None),
        'img-src': getattr(settings, 'CSP_IMG_SRC', None),
        'manifest-src': getattr(settings, 'CSP_MANIFEST_SRC', None),
        'media-src': getattr(settings, 'CSP_MEDIA_SRC', None),
        'prefetch-src': getattr(settings, 'CSP_PREFETCH_SRC', None),
        'worker-src': getattr(settings, 'CSP_WORKER_SRC', None),
        # Document Directives
        'base-uri': getattr(settings, 'CSP_BASE_URI', None),
        'plugin-types': getattr(settings, 'CSP_PLUGIN_TYPES', None),
        'sandbox': getattr(settings, 'CSP_SANDBOX', None),
        # Navigation Directives
        'form-action': getattr(settings, 'CSP_FORM_ACTION', None),
        'frame-ancestors': getattr(settings, 'CSP_FRAME_ANCESTORS', None),
        'navigate-to': getattr(settings, 'CSP_NAVIGATE_TO', None),
        # Reporting Directives
        'report-uri': getattr(settings, 'CSP_REPORT_URI', None),
        'report-to': getattr(settings, 'CSP_REPORT_TO', None),
        'require-sri-for': getattr(settings, 'CSP_REQUIRE_SRI_FOR', None),
        #trusted Types Directives
        'require-trusted-types-for': getattr(
            settings,
            'CSP_REQUIRE_TRUSTED_TYPES_FOR', None),
        'trusted-types': getattr(settings, 'CSP_TRUSTED_TYPES', None),
        # Other Directives
        'upgrade-insecure-requests': getattr(
            settings, 'CSP_UPGRADE_INSECURE_REQUESTS', False),
        'block-all-mixed-content': getattr(
            settings, 'CSP_BLOCK_ALL_MIXED_CONTENT', False),
    }


def build_policy(config=None, update=None, replace=None, nonce=None):
    """Builds the policy as a string from the settings."""

    if config is None:
        config = from_settings()
        # Be careful, don't mutate config as it could be from settings

    update = update if update is not None else {}
    replace = replace if replace is not None else {}
    csp = {}

    for k in set(chain(config, replace)):
        if k in replace:
            v = replace[k]
        else:
            v = config[k]
        if v is not None:
            v = copy.copy(v)
            if not isinstance(v, (list, tuple)):
                v = (v,)
            csp[k] = v

    for k, v in update.items():
        if v is not None:
            if not isinstance(v, (list, tuple)):
                v = (v,)
            if csp.get(k) is None:
                csp[k] = v
            else:
                csp[k] += tuple(v)

    report_uri = csp.pop('report-uri', None)

    policy_parts = {}
    for key, value in csp.items():
        # flag directives with an empty directive value
        if len(value) and value[0] is True:
            policy_parts[key] = ''
        elif len(value) and value[0] is False:
            pass
        else:  # directives with many values like src lists
            policy_parts[key] = ' '.join(value)

        if key == 'child-src':
            warnings.warn(CHILD_SRC_DEPRECATION_WARNING, DeprecationWarning)

    if report_uri:
        report_uri = map(force_str, report_uri)
        policy_parts['report-uri'] = ' '.join(report_uri)

    if nonce:
        include_nonce_in = getattr(settings, 'CSP_INCLUDE_NONCE_IN',
                                   ['default-src'])
        for section in include_nonce_in:
            policy = policy_parts.get(section, '')
            policy_parts[section] = ("%s %s" %
                                     (policy, "'nonce-%s'" % nonce)).strip()

    return '; '.join(['{} {}'.format(k, val).strip()
                      for k, val in policy_parts.items()])


def _default_attr_mapper(attr_name, val):
    if val:
        return ' {}="{}"'.format(attr_name, val)
    else:
        return ''


def _bool_attr_mapper(attr_name, val):
    # Only return the bare word if the value is truthy
    # ie - defer=False should actually return an empty string
    if val:
        return ' {}'.format(attr_name)
    else:
        return ''


def _async_attr_mapper(attr_name, val):
    """The `async` attribute works slightly different than the other bool
    attributes. It can be set explicitly to `false` with no surrounding quotes
    according to the spec."""
    if val in [False, 'False']:
        return ' {}=false'.format(attr_name)
    elif val:
        return ' {}'.format(attr_name)
    else:
        return ''


# Allow per-attribute customization of returned string template
SCRIPT_ATTRS = OrderedDict()
SCRIPT_ATTRS['nonce'] = _default_attr_mapper
SCRIPT_ATTRS['id'] = _default_attr_mapper
SCRIPT_ATTRS['src'] = _default_attr_mapper
SCRIPT_ATTRS['type'] = _default_attr_mapper
SCRIPT_ATTRS['async'] = _async_attr_mapper
SCRIPT_ATTRS['defer'] = _bool_attr_mapper
SCRIPT_ATTRS['integrity'] = _default_attr_mapper
SCRIPT_ATTRS['nomodule'] = _bool_attr_mapper

# Generates an interpolatable string of valid attrs eg - '{nonce}{id}...'
ATTR_FORMAT_STR = ''.join(['{{{}}}'.format(a) for a in SCRIPT_ATTRS])


def _unwrap_script(text):
    """Extract content defined between script tags"""
    matches = re.search(r'<script[\s|\S]*>([\s|\S]+?)</script>', text)
    if matches and len(matches.groups()):
        return matches.group(1).strip()

    return text


def build_script_tag(content=None, **kwargs):
    data = {}
    # Iterate all possible script attrs instead of kwargs to make
    # interpolation as easy as possible below
    for attr_name, mapper in SCRIPT_ATTRS.items():
        data[attr_name] = mapper(attr_name, kwargs.get(attr_name))

    # Don't render block contents if the script has a 'src' attribute
    c = _unwrap_script(content) if content and not kwargs.get('src') else ''
    attrs = ATTR_FORMAT_STR.format(**data).rstrip()
    return ('<script{}>{}</script>'.format(attrs, c).strip())



from __future__ import absolute_import

import pytest
import six
from django.conf import settings
from django.test.utils import override_settings
from django.utils.functional import lazy

from csp.utils import build_policy


def policy_eq(a, b, msg='%r != %r'):
    parts_a = sorted(a.split('; '))
    parts_b = sorted(b.split('; '))
    assert parts_a == parts_b, msg % (a, b)


def test_empty_policy():
    policy = build_policy()
    assert "default-src 'self'" == policy


def literal(s):
    return s


lazy_literal = lazy(literal, six.text_type)


@override_settings(CSP_DEFAULT_SRC=['example.com', 'example2.com'])
def test_default_src():
    policy = build_policy()
    assert 'default-src example.com example2.com' == policy


@override_settings(CSP_SCRIPT_SRC=['example.com'])
def test_script_src():
    policy = build_policy()
    policy_eq("default-src 'self'; script-src example.com", policy)


@override_settings(CSP_SCRIPT_SRC_ATTR=['example.com'])
def test_script_src_attr():
    policy = build_policy()
    policy_eq("default-src 'self'; script-src-attr example.com", policy)


@override_settings(CSP_SCRIPT_SRC_ELEM=['example.com'])
def test_script_src_elem():
    policy = build_policy()
    policy_eq("default-src 'self'; script-src-elem example.com", policy)


@override_settings(CSP_OBJECT_SRC=['example.com'])
def test_object_src():
    policy = build_policy()
    policy_eq("default-src 'self'; object-src example.com", policy)


@override_settings(CSP_PREFETCH_SRC=['example.com'])
def test_prefetch_src():
    policy = build_policy()
    policy_eq("default-src 'self'; prefetch-src example.com", policy)


@override_settings(CSP_STYLE_SRC=['example.com'])
def test_style_src():
    policy = build_policy()
    policy_eq("default-src 'self'; style-src example.com", policy)


@override_settings(CSP_STYLE_SRC_ATTR=['example.com'])
def test_style_src_attr():
    policy = build_policy()
    policy_eq("default-src 'self'; style-src-attr example.com", policy)


@override_settings(CSP_STYLE_SRC_ELEM=['example.com'])
def test_style_src_elem():
    policy = build_policy()
    policy_eq("default-src 'self'; style-src-elem example.com", policy)


@override_settings(CSP_IMG_SRC=['example.com'])
def test_img_src():
    policy = build_policy()
    policy_eq("default-src 'self'; img-src example.com", policy)


@override_settings(CSP_MEDIA_SRC=['example.com'])
def test_media_src():
    policy = build_policy()
    policy_eq("default-src 'self'; media-src example.com", policy)


@override_settings(CSP_FRAME_SRC=['example.com'])
def test_frame_src():
    policy = build_policy()
    policy_eq("default-src 'self'; frame-src example.com", policy)


@override_settings(CSP_FONT_SRC=['example.com'])
def test_font_src():
    policy = build_policy()
    policy_eq("default-src 'self'; font-src example.com", policy)


@override_settings(CSP_CONNECT_SRC=['example.com'])
def test_connect_src():
    policy = build_policy()
    policy_eq("default-src 'self'; connect-src example.com", policy)


@override_settings(CSP_SANDBOX=['allow-scripts'])
def test_sandbox():
    policy = build_policy()
    policy_eq("default-src 'self'; sandbox allow-scripts", policy)


@override_settings(CSP_SANDBOX=[])
def test_sandbox_empty():
    policy = build_policy()
    policy_eq("default-src 'self'; sandbox", policy)


@override_settings(CSP_REPORT_URI='/foo')
def test_report_uri():
    policy = build_policy()
    policy_eq("default-src 'self'; report-uri /foo", policy)


@override_settings(CSP_REPORT_URI=lazy_literal('/foo'))
def test_report_uri_lazy():
    policy = build_policy()
    policy_eq("default-src 'self'; report-uri /foo", policy)


@override_settings(CSP_REPORT_TO='some_endpoint')
def test_report_to():
    policy = build_policy()
    policy_eq("default-src 'self'; report-to some_endpoint",
              policy)


@override_settings(CSP_IMG_SRC=['example.com'])
def test_update_img():
    policy = build_policy(update={'img-src': 'example2.com'})
    policy_eq("default-src 'self'; img-src example.com example2.com",
              policy)


def test_update_missing_setting():
    """update should work even if the setting is not defined."""
    policy = build_policy(update={'img-src': 'example.com'})
    policy_eq("default-src 'self'; img-src example.com", policy)


@override_settings(CSP_IMG_SRC=['example.com'])
def test_replace_img():
    policy = build_policy(replace={'img-src': 'example2.com'})
    policy_eq("default-src 'self'; img-src example2.com", policy)


def test_replace_missing_setting():
    """replace should work even if the setting is not defined."""
    policy = build_policy(replace={'img-src': 'example.com'})
    policy_eq("default-src 'self'; img-src example.com", policy)


def test_config():
    policy = build_policy(
        config={'default-src': ["'none'"], 'img-src': ["'self'"]})
    policy_eq("default-src 'none'; img-src 'self'", policy)


@override_settings(CSP_IMG_SRC=('example.com',))
def test_update_string():
    """
    GitHub issue #40 - given project settings as a tuple, and
    an update/replace with a string, concatenate correctly.
    """
    policy = build_policy(update={'img-src': 'example2.com'})
    policy_eq("default-src 'self'; img-src example.com example2.com",
              policy)


@override_settings(CSP_IMG_SRC=('example.com',))
def test_replace_string():
    """
    Demonstrate that GitHub issue #40 doesn't affect replacements
    """
    policy = build_policy(replace={'img-src': 'example2.com'})
    policy_eq("default-src 'self'; img-src example2.com",
              policy)


@override_settings(CSP_FORM_ACTION=['example.com'])
def test_form_action():
    policy = build_policy()
    policy_eq("default-src 'self'; form-action example.com", policy)


@override_settings(CSP_BASE_URI=['example.com'])
def test_base_uri():
    policy = build_policy()
    policy_eq("default-src 'self'; base-uri example.com", policy)


@override_settings(CSP_CHILD_SRC=['example.com'])
def test_child_src():
    with pytest.warns(DeprecationWarning):
        policy = build_policy()
        policy_eq("default-src 'self'; child-src example.com", policy)


@override_settings(CSP_FRAME_ANCESTORS=['example.com'])
def test_frame_ancestors():
    policy = build_policy()
    policy_eq("default-src 'self'; frame-ancestors example.com", policy)


@override_settings(CSP_NAVIGATE_TO=['example.com'])
def test_navigate_to():
    policy = build_policy()
    policy_eq("default-src 'self'; navigate-to example.com", policy)


@override_settings(CSP_MANIFEST_SRC=['example.com'])
def test_manifest_src():
    policy = build_policy()
    policy_eq("default-src 'self'; manifest-src example.com", policy)


@override_settings(CSP_WORKER_SRC=['example.com'])
def test_worker_src():
    policy = build_policy()
    policy_eq("default-src 'self'; worker-src example.com", policy)


@override_settings(CSP_PLUGIN_TYPES=['application/pdf'])
def test_plugin_types():
    policy = build_policy()
    policy_eq("default-src 'self'; plugin-types application/pdf", policy)


@override_settings(CSP_REQUIRE_SRI_FOR=['script'])
def test_require_sri_for():
    policy = build_policy()
    policy_eq("default-src 'self'; require-sri-for script", policy)


@override_settings(CSP_REQUIRE_TRUSTED_TYPES_FOR=["'script'"])
def test_require_trusted_types_for():
    policy = build_policy()
    policy_eq("default-src 'self'; require-trusted-types-for 'script'", policy)


@override_settings(CSP_TRUSTED_TYPES=["strictPolicy", "laxPolicy",
                                      "default", "'allow-duplicates'"])
def test_trusted_types():
    policy = build_policy()
    policy_eq("default-src 'self'; trusted-types strictPolicy laxPolicy "
              + "default 'allow-duplicates'", policy)


@override_settings(CSP_UPGRADE_INSECURE_REQUESTS=True)
def test_upgrade_insecure_requests():
    policy = build_policy()
    policy_eq("default-src 'self'; upgrade-insecure-requests", policy)


@override_settings(CSP_BLOCK_ALL_MIXED_CONTENT=True)
def test_block_all_mixed_content():
    policy = build_policy()
    policy_eq("default-src 'self'; block-all-mixed-content", policy)


def test_nonce():
    policy = build_policy(nonce='abc123')
    policy_eq("default-src 'self' 'nonce-abc123'", policy)


@override_settings(CSP_INCLUDE_NONCE_IN=['script-src', 'style-src'])
def test_nonce_include_in():
    policy = build_policy(nonce='abc123')
    policy_eq(("default-src 'self'; "
               "script-src 'nonce-abc123'; "
               "style-src 'nonce-abc123'"), policy)


@override_settings()
def test_nonce_include_in_absent():
    del settings.CSP_INCLUDE_NONCE_IN
    policy = build_policy(nonce='abc123')
    policy_eq("default-src 'self' 'nonce-abc123'", policy)



from __future__ import absolute_import

from django import template
from django.template.base import token_kwargs

from csp.utils import build_script_tag

register = template.Library()


def _unquote(s):
    """Helper func that strips single and double quotes from inside strings"""
    return s.replace('"', '').replace("'", "")


@register.tag(name='script')
def script(parser, token):
    # Parse out any keyword args
    token_args = token.split_contents()
    kwargs = token_kwargs(token_args[1:], parser)

    nodelist = parser.parse(('endscript',))
    parser.delete_first_token()

    return NonceScriptNode(nodelist, **kwargs)


class NonceScriptNode(template.Node):
    def __init__(self, nodelist, **kwargs):
        self.nodelist = nodelist
        self.script_attrs = {}
        for k, v in kwargs.items():
            self.script_attrs[k] = self._get_token_value(v)

    def _get_token_value(self, t):
        return _unquote(t.token) if getattr(t, 'token', None) else None

    def render(self, context):
        output = self.nodelist.render(context).strip()
        request = context.get('request')
        nonce = request.csp_nonce if hasattr(request, 'csp_nonce') else ''
        self.script_attrs.update({'nonce': nonce, 'content': output})

        return build_script_tag(**self.script_attrs)



from django.http import HttpResponse
from django.test import RequestFactory
from django.test.utils import override_settings

from csp.decorators import csp, csp_replace, csp_update, csp_exempt
from csp.middleware import CSPMiddleware


REQUEST = RequestFactory().get('/')
mw = CSPMiddleware()


def test_csp_exempt():
    @csp_exempt
    def view(request):
        return HttpResponse()
    response = view(REQUEST)
    assert response._csp_exempt


@override_settings(CSP_IMG_SRC=['foo.com'])
def test_csp_update():
    def view_without_decorator(request):
        return HttpResponse()
    response = view_without_decorator(REQUEST)
    mw.process_response(REQUEST, response)
    policy_list = sorted(response['Content-Security-Policy'].split("; "))
    assert policy_list == ["default-src 'self'", "img-src foo.com"]

    @csp_update(IMG_SRC='bar.com')
    def view_with_decorator(request):
        return HttpResponse()
    response = view_with_decorator(REQUEST)
    assert response._csp_update == {'img-src': 'bar.com'}
    mw.process_response(REQUEST, response)
    policy_list = sorted(response['Content-Security-Policy'].split("; "))
    assert policy_list == ["default-src 'self'", "img-src foo.com bar.com"]

    response = view_without_decorator(REQUEST)
    mw.process_response(REQUEST, response)
    policy_list = sorted(response['Content-Security-Policy'].split("; "))
    assert policy_list == ["default-src 'self'", "img-src foo.com"]


@override_settings(CSP_IMG_SRC=['foo.com'])
def test_csp_replace():
    def view_without_decorator(request):
        return HttpResponse()
    response = view_without_decorator(REQUEST)
    mw.process_response(REQUEST, response)
    policy_list = sorted(response['Content-Security-Policy'].split("; "))
    assert policy_list == ["default-src 'self'", "img-src foo.com"]

    @csp_replace(IMG_SRC='bar.com')
    def view_with_decorator(request):
        return HttpResponse()
    response = view_with_decorator(REQUEST)
    assert response._csp_replace == {'img-src': 'bar.com'}
    mw.process_response(REQUEST, response)
    policy_list = sorted(response['Content-Security-Policy'].split("; "))
    assert policy_list == ["default-src 'self'", "img-src bar.com"]

    response = view_without_decorator(REQUEST)
    mw.process_response(REQUEST, response)
    policy_list = sorted(response['Content-Security-Policy'].split("; "))
    assert policy_list == ["default-src 'self'", "img-src foo.com"]

    @csp_replace(IMG_SRC=None)
    def view_removing_directive(request):
        return HttpResponse()
    response = view_removing_directive(REQUEST)
    mw.process_response(REQUEST, response)
    policy_list = sorted(response["Content-Security-Policy"].split("; "))
    assert policy_list == ["default-src 'self'"]


def test_csp():
    def view_without_decorator(request):
        return HttpResponse()
    response = view_without_decorator(REQUEST)
    mw.process_response(REQUEST, response)
    policy_list = sorted(response['Content-Security-Policy'].split("; "))
    assert policy_list == ["default-src 'self'"]

    @csp(IMG_SRC=['foo.com'], FONT_SRC=['bar.com'])
    def view_with_decorator(request):
        return HttpResponse()
    response = view_with_decorator(REQUEST)
    assert response._csp_config == \
        {'img-src': ['foo.com'], 'font-src': ['bar.com']}
    mw.process_response(REQUEST, response)
    policy_list = sorted(response['Content-Security-Policy'].split("; "))
    assert policy_list == ["font-src bar.com", "img-src foo.com"]

    response = view_without_decorator(REQUEST)
    mw.process_response(REQUEST, response)
    policy_list = sorted(response['Content-Security-Policy'].split("; "))
    assert policy_list == ["default-src 'self'"]


def test_csp_string_values():
    # Test backwards compatibility where values were strings
    @csp(IMG_SRC='foo.com', FONT_SRC='bar.com')
    def view_with_decorator(request):
        return HttpResponse()
    response = view_with_decorator(REQUEST)
    assert response._csp_config == \
        {'img-src': ['foo.com'], 'font-src': ['bar.com']}
    mw.process_response(REQUEST, response)
    policy_list = sorted(response['Content-Security-Policy'].split("; "))
    assert policy_list == ["font-src bar.com", "img-src foo.com"]



from csp.tests.utils import ScriptExtensionTestBase


class TestJinjaExtension(ScriptExtensionTestBase):

    def test_script_tag_injects_nonce(self):
        tpl = """
            {% script %}
                var hello='world';
            {% endscript %}
        """

        expected = ("""<script nonce="{}">var hello='world';</script>""")
        self.assert_template_eq(*self.process_templates(tpl, expected))

    def test_script_with_src_ignores_body(self):
        tpl = """
            {% script src="foo" %}
                var hello='world';
            {% endscript %}
        """

        expected = """<script nonce="{}" src="foo"></script>"""

        self.assert_template_eq(*self.process_templates(tpl, expected))

    def test_script_tag_sets_attrs_correctly(self):
        tpl = """
            {% script id='jeff' defer=True %}
                var hello='world';
            {% endscript %}
            """
        expected = """
            <script nonce="{}" id="jeff" defer>
                var hello='world';
            </script>"""

        self.assert_template_eq(*self.process_templates(tpl, expected))

    def test_async_attribute_with_falsey(self):
        tpl = """
            {% script id="jeff" async=False %}
                var hello='world';
            {% endscript %}"""

        expected = ('<script nonce="{}" id="jeff" async=false>'
                    'var hello=\'world\';'
                    '</script>')

        self.assert_template_eq(*self.process_templates(tpl, expected))

    def test_async_attribute_with_truthy(self):
        tpl = """
            {% script id="jeff" async=True %}
                var hello='world';
            {% endscript %}"""

        expected = ('<script nonce="{}" id="jeff" async>'
                    'var hello=\'world\';'
                    '</script>')

        self.assert_template_eq(*self.process_templates(tpl, expected))

    def test_nested_script_tags_are_removed(self):
        """Let users wrap their code in script tags for the sake of their
           development environment"""
        tpl = """
            {% script type="application/javascript" id="jeff" defer=True%}
                <script type="text/javascript">
                var hello='world';
                </script>
            {% endscript %}"""

        expected = (
            '<script'
            ' nonce="{}" id="jeff" type="application/javascript" defer>'
            'var hello=\'world\';</script>')

        self.assert_template_eq(*self.process_templates(tpl, expected))



import random

from django.conf import settings

from csp.middleware import CSPMiddleware
from csp.utils import build_policy


class RateLimitedCSPMiddleware(CSPMiddleware):
    """A CSP middleware that rate-limits the number of violation reports sent
    to report-uri by excluding it from some requests."""

    def build_policy(self, request, response):
        config = getattr(response, '_csp_config', None)
        update = getattr(response, '_csp_update', None)
        replace = getattr(response, '_csp_replace', {})
        nonce = getattr(request, '_csp_nonce', None)

        report_percentage = getattr(settings, 'CSP_REPORT_PERCENTAGE')
        include_report_uri = random.random() < report_percentage
        if not include_report_uri:
            replace['report-uri'] = None

        return build_policy(config=config, update=update, replace=replace,
                            nonce=nonce)



from __future__ import absolute_import

from jinja2 import nodes
from jinja2.ext import Extension

from csp.utils import SCRIPT_ATTRS, build_script_tag


class NoncedScript(Extension):
    # a set of names that trigger the extension.
    tags = set(['script'])

    def parse(self, parser):
        # the first token is the token that started the tag.  In our case
        # we only listen to ``'script'`` so this will be a name token with
        # `script` as value.  We get the line number so that we can give
        # that line number to the nodes we create by hand.
        lineno = next(parser.stream).lineno

        # Get the current context and pass along
        kwargs = [nodes.Keyword('ctx', nodes.ContextReference())]

        # Parse until we are done with optional script tag attributes
        while parser.stream.current.value in SCRIPT_ATTRS:
            attr_name = parser.stream.current.value
            parser.stream.skip(2)
            kwargs.append(
                nodes.Keyword(attr_name, parser.parse_expression()))

        # now we parse the body of the script block up to `endscript` and
        # drop the needle (which would always be `endscript` in that case)
        body = parser.parse_statements(['name:endscript'], drop_needle=True)

        # now return a `CallBlock` node that calls our _render_script
        # helper method on this extension.
        return nodes.CallBlock(
            self.call_method('_render_script', kwargs=kwargs),
            [], [], body).set_lineno(lineno)

    def _render_script(self, caller, **kwargs):
        ctx = kwargs.pop('ctx')
        request = ctx.get('request')
        kwargs['nonce'] = request.csp_nonce
        kwargs['content'] = caller().strip()

        return build_script_tag(**kwargs)



from django.http import HttpResponse
from django.test import RequestFactory
from django.test.utils import override_settings

from csp.contrib.rate_limiting import RateLimitedCSPMiddleware


HEADER = 'Content-Security-Policy'
mw = RateLimitedCSPMiddleware()
rf = RequestFactory()


@override_settings(CSP_REPORT_PERCENTAGE=0.1, CSP_REPORT_URI='x')
def test_report_percentage():
    times_seen = 0
    for _ in range(5000):
        request = rf.get('/')
        response = HttpResponse()
        mw.process_response(request, response)
        if 'report-uri' in response[HEADER]:
            times_seen += 1
    # Roughly 10%
    assert 400 <= times_seen <= 600



from functools import wraps


def csp_exempt(f):
    @wraps(f)
    def _wrapped(*a, **kw):
        r = f(*a, **kw)
        r._csp_exempt = True
        return r
    return _wrapped


def csp_update(**kwargs):
    update = dict((k.lower().replace('_', '-'), v) for k, v in kwargs.items())

    def decorator(f):
        @wraps(f)
        def _wrapped(*a, **kw):
            r = f(*a, **kw)
            r._csp_update = update
            return r
        return _wrapped
    return decorator


def csp_replace(**kwargs):
    replace = dict((k.lower().replace('_', '-'), v) for k, v in kwargs.items())

    def decorator(f):
        @wraps(f)
        def _wrapped(*a, **kw):
            r = f(*a, **kw)
            r._csp_replace = replace
            return r
        return _wrapped
    return decorator


def csp(**kwargs):
    config = dict(
        (k.lower().replace('_', '-'), [v] if isinstance(v, str) else v)
        for k, v
        in kwargs.items()
    )

    def decorator(f):
        @wraps(f)
        def _wrapped(*a, **kw):
            r = f(*a, **kw)
            r._csp_config = config
            return r
        return _wrapped
    return decorator



import sys
import os
import codecs
from setuptools import setup, find_packages


version = '3.7'


if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    os.system('python setup.py bdist_wheel upload')
    print('You probably want to also tag the version now:')
    print('  git tag -a %s -m "version %s"' % (version, version))
    print('  git push --tags')
    sys.exit()


def read(*parts):
    filename = os.path.join(os.path.dirname(__file__), *parts)
    with codecs.open(filename, encoding='utf-8') as fp:
        return fp.read()


install_requires = [
    'Django>=1.8',
]

jinja2_requires = [
    'jinja2>=2.9.6',
]

test_requires = [
    'pytest<4.0',
    'pytest-django',
    'pytest-flakes==1.0.1',
    'pytest-pep8==1.0.6',
    'pep8==1.4.6',
    'mock==1.0.1',
    'six==1.12.0',
]

test_requires += jinja2_requires


setup(
    name='django_csp',
    version=version,
    description='Django Content Security Policy support.',
    long_description=read('README.rst'),
    author='James Socol',
    author_email='me@jamessocol.com',
    maintainer='Christopher Grebs',
    maintainer_email='cg@webshox.org',
    url='http://github.com/mozilla/django-csp',
    license='BSD',
    packages=find_packages(),
    install_requires=install_requires,
    extras_require={
        'tests': test_requires,
        'jinja2': jinja2_requires,
    },
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Environment :: Web Environment :: Mozilla',
        'Programming Language :: Python',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Programming Language :: Python :: Implementation :: CPython',
        'Framework :: Django',
    ]
)



from django.http import (
    HttpResponse,
    HttpResponseServerError,
    HttpResponseNotFound,
)
from django.test import RequestFactory
from django.test.utils import override_settings

from csp.middleware import CSPMiddleware


HEADER = 'Content-Security-Policy'
mw = CSPMiddleware()
rf = RequestFactory()


def test_add_header():
    request = rf.get('/')
    response = HttpResponse()
    mw.process_response(request, response)
    assert HEADER in response


def test_exempt():
    request = rf.get('/')
    response = HttpResponse()
    response._csp_exempt = True
    mw.process_response(request, response)
    assert HEADER not in response


@override_settings(CSP_EXCLUDE_URL_PREFIXES=('/inlines-r-us'))
def text_exclude():
    request = rf.get('/inlines-r-us/foo')
    response = HttpResponse()
    mw.process_response(request, response)
    assert HEADER not in response


@override_settings(CSP_REPORT_ONLY=True)
def test_report_only():
    request = rf.get('/')
    response = HttpResponse()
    mw.process_response(request, response)
    assert HEADER not in response
    assert HEADER + '-Report-Only' in response


def test_dont_replace():
    request = rf.get('/')
    response = HttpResponse()
    response[HEADER] = 'default-src example.com'
    mw.process_response(request, response)
    assert response[HEADER] == 'default-src example.com'


def test_use_config():
    request = rf.get('/')
    response = HttpResponse()
    response._csp_config = {'default-src': ['example.com']}
    mw.process_response(request, response)
    assert response[HEADER] == 'default-src example.com'


def test_use_update():
    request = rf.get('/')
    response = HttpResponse()
    response._csp_update = {'default-src': ['example.com']}
    mw.process_response(request, response)
    assert response[HEADER] == "default-src 'self' example.com"


@override_settings(CSP_IMG_SRC=['foo.com'])
def test_use_replace():
    request = rf.get('/')
    response = HttpResponse()
    response._csp_replace = {'img-src': ['bar.com']}
    mw.process_response(request, response)
    policy_list = sorted(response[HEADER].split('; '))
    assert policy_list == ["default-src 'self'", "img-src bar.com"]


@override_settings(DEBUG=True)
def test_debug_errors_exempt():
    request = rf.get('/')
    response = HttpResponseServerError()
    mw.process_response(request, response)
    assert HEADER not in response


@override_settings(DEBUG=True)
def test_debug_notfound_exempt():
    request = rf.get('/')
    response = HttpResponseNotFound()
    mw.process_response(request, response)
    assert HEADER not in response


def test_nonce_created_when_accessed():
    request = rf.get('/')
    mw.process_request(request)
    nonce = str(request.csp_nonce)
    response = HttpResponse()
    mw.process_response(request, response)
    assert nonce in response[HEADER]


def test_no_nonce_when_not_accessed():
    request = rf.get('/')
    mw.process_request(request)
    response = HttpResponse()
    mw.process_response(request, response)
    assert 'nonce-' not in response[HEADER]


def test_nonce_regenerated_on_new_request():
    request1 = rf.get('/')
    request2 = rf.get('/')
    mw.process_request(request1)
    mw.process_request(request2)
    nonce1 = str(request1.csp_nonce)
    nonce2 = str(request2.csp_nonce)
    assert request1.csp_nonce != request2.csp_nonce

    response1 = HttpResponse()
    response2 = HttpResponse()
    mw.process_response(request1, response1)
    mw.process_response(request2, response2)
    assert nonce1 not in response2[HEADER]
    assert nonce2 not in response1[HEADER]


@override_settings(CSP_INCLUDE_NONCE_IN=[])
def test_no_nonce_when_disabled_by_settings():
    request = rf.get('/')
    mw.process_request(request)
    nonce = str(request.csp_nonce)
    response = HttpResponse()
    mw.process_response(request, response)
    assert nonce not in response[HEADER]



from __future__ import absolute_import

import os
import base64
from functools import partial

from django.conf import settings
from django.utils.functional import SimpleLazyObject

try:
    from django.utils.six.moves import http_client
except ImportError:
    # django 3.x removed six
    import http.client as http_client

try:
    from django.utils.deprecation import MiddlewareMixin
except ImportError:
    class MiddlewareMixin(object):
        """
        If this middleware doesn't exist, this is an older version of django
        and we don't need it.
        """
        pass

from csp.utils import build_policy


class CSPMiddleware(MiddlewareMixin):
    """
    Implements the Content-Security-Policy response header, which
    conforming user-agents can use to restrict the permitted sources
    of various content.

    See http://www.w3.org/TR/CSP/

    """
    def _make_nonce(self, request):
        # Ensure that any subsequent calls to request.csp_nonce return the
        # same value
        if not getattr(request, '_csp_nonce', None):
            request._csp_nonce = (
                base64
                .b64encode(os.urandom(16))
                .decode("ascii")
            )
        return request._csp_nonce

    def process_request(self, request):
        nonce = partial(self._make_nonce, request)
        request.csp_nonce = SimpleLazyObject(nonce)

    def process_response(self, request, response):
        if getattr(response, '_csp_exempt', False):
            return response

        # Check for ignored path prefix.
        prefixes = getattr(settings, 'CSP_EXCLUDE_URL_PREFIXES', ())
        if request.path_info.startswith(prefixes):
            return response

        # Check for debug view
        status_code = response.status_code
        exempted_debug_codes = (
            http_client.INTERNAL_SERVER_ERROR,
            http_client.NOT_FOUND,
        )
        if status_code in exempted_debug_codes and settings.DEBUG:
            return response

        header = 'Content-Security-Policy'
        if getattr(settings, 'CSP_REPORT_ONLY', False):
            header += '-Report-Only'

        if header in response:
            # Don't overwrite existing headers.
            return response

        response[header] = self.build_policy(request, response)

        return response

    def build_policy(self, request, response):
        config = getattr(response, '_csp_config', None)
        update = getattr(response, '_csp_update', None)
        replace = getattr(response, '_csp_replace', None)
        nonce = getattr(request, '_csp_nonce', None)
        return build_policy(config=config, update=update, replace=replace,
                            nonce=nonce)
